"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DeployEasy = void 0;
const vscode_api_1 = require("../vscode_api");
const base_util_1 = require("../utils/base_util");
const file_util_1 = require("../utils/file_util");
const ssh2tools_1 = require("../ssh2tool/ssh2tools");
const global_constants_1 = require("../consts/global_constants");
const polling_util_1 = require("../utils/polling_util");
const util_1 = require("./util");
const path_1 = require("path");
const vscode_api_2 = require("../vscode_api/vscode_api");
/**
 * 一键部署类
 * @params sourceDir 默认是./script
 * @params workDir 服务器存放上传文件的工作目录
 * @params shellName 脚本名
 * @params packExistCmd 检查包存在的命令
 * @params logName 日志名
 * @params terminalName 默认是host:port
 * @params conn 连接了服务器的实例
 * @params templateName 默认是install_template.sh
 */
class DeployEasy {
    constructor({ conn, sourceDir = global_constants_1.SOURCE_DIR, workDir = global_constants_1.TMP_DIR, shellName = global_constants_1.INSTALL_SHELL_NAME, packExistCmd = global_constants_1.PACK_EXIST_CMD, logName = global_constants_1.INSTALL_SHELL_LOG, terminalName, panel }) {
        this.sourceDir = util_1.trans2Dir(sourceDir);
        this.workDir = util_1.trans2Dir(workDir);
        this.shellName = shellName;
        this.packExistCmd = packExistCmd;
        this.logName = logName;
        this.conn = conn;
        const { host, port } = conn.getServerInfo();
        this.terminalName = terminalName || `${host}:${port}`;
        this.terminal = null;
        this.output = null;
        this.panel = panel;
        // 日志路径，本地脚本路径，远端脚本路径都是可计算的
        Object.defineProperties(this, {
            logPath: {
                get() {
                    return this.workDir + this.logName;
                }
            },
            localShellFile: {
                get() {
                    return path_1.join(vscode_api_1.default.extensionPath, this.sourceDir, this.shellName);
                }
            },
            remoteShellFile: {
                get() {
                    return this.workDir + this.shellName;
                }
            }
        });
    }
    // 保存x2openEuler用户密码
    savePwd(pwd) {
        this.pwd = pwd;
    }
    // 获取一键部署用的终端实例
    getDeployTerminal() {
        return this.terminal;
    }
    runInstall() {
        return __awaiter(this, void 0, void 0, function* () {
            const exeShell = `bash ${this.remoteShellFile}`;
            const outputName = this.terminalName;
            vscode_api_1.default.writeLog('debug', `执行的install命令：${exeShell}`);
            this.panel.stored.output = vscode_api_1.default.writeOutput('', 'info', outputName, true, true);
            this.output = this.panel.stored.output;
            this.conn.exeCMDRemote(exeShell, {
                onData: (stdout) => {
                    // 在output中回显
                    vscode_api_1.default.writeOutput(stdout, 'info', outputName, false, false);
                    // 在运行日志中记录一下
                    vscode_api_1.default.writeLog('debug', stdout);
                }
            });
        });
    }
    x2openEulerExists() {
        return __awaiter(this, void 0, void 0, function* () {
            let version;
            try {
                for (const packExistCmd of this.packExistCmd) {
                    yield this.conn.exeCMDRemote(packExistCmd, {
                        onData: (res) => {
                            try {
                                const splitRes = base_util_1.getObjectFromString(res);
                                if (splitRes.Version) {
                                    version = splitRes.Version;
                                }
                            }
                            catch (e) {
                                // 不用做啥
                            }
                        }
                    });
                    if (version)
                        break;
                }
                return version;
            }
            catch (e) {
                throw e;
            }
        });
    }
    // 查看安装包名
    x2openEulerPackName() {
        return __awaiter(this, void 0, void 0, function* () {
            let packName;
            try {
                yield this.conn.exeCMDRemote(global_constants_1.INSTALLED_PACKNAME, {
                    onData: (res) => {
                        packName = res;
                    }
                });
            }
            catch (e) {
                throw e;
            }
            return packName;
        });
    }
    /**
     * 联网下载流程：包括下载和上传
     * @param url 安装包的网络地址url
     */
    uploadThroughInternet(url) {
        return __awaiter(this, void 0, void 0, function* () {
            const fileName = file_util_1.getPathName(url);
            const { host, port } = this.getServerData();
            const serverMark = `${host}@${port}`;
            const localDir = path_1.join(vscode_api_1.default.extensionPath, `./tmp_${serverMark}_download`);
            yield file_util_1.mkDirAsync(localDir, { recursive: true });
            const localPath = path_1.join(localDir, fileName);
            const remotePath = this.workDir + fileName;
            try {
                yield util_1.downloadResourceWithProgress(url, localPath);
                yield util_1.uploadFileWithProgress(this.conn, localPath, remotePath);
            }
            catch (e) {
                throw e;
            }
            finally {
                // 无论上传到服务器成功或失败都清除本地的文件
                if (yield file_util_1.fileExistAsync(localPath)) {
                    yield file_util_1.unLinkAsync(localPath);
                }
                yield file_util_1.rmDirAsync(localDir);
            }
        });
    }
    /**
     * 获取部署工具包
     * @returns string
     */
    getDeployPackage() {
        return __awaiter(this, void 0, void 0, function* () {
            const prefixName = global_constants_1.PACKAGE_URL;
            let deployPackage;
            try {
                let arch = yield this.conn.checkInfo(global_constants_1.CHECK_ARCH);
                let operateSys = base_util_1.getObjectFromString(yield this.conn.checkInfo(global_constants_1.CHECK_OPERATE_SYS), { strChar: '\n', keyChar: '=' });
                let systemId = operateSys.ID.replace(/"/g, '');
                let versionId = operateSys.VERSION_ID.replace(/"/g, '');
                if (versionId === '6') {
                    deployPackage = `${prefixName}x86_64/Packages/${global_constants_1.PACKAGE_PREFIX_NAME}el6.x86_64.rpm`;
                }
                else {
                    deployPackage = `${prefixName}${arch}/Packages/${global_constants_1.PACKAGE_PREFIX_NAME}${arch}.rpm`;
                }
            }
            catch (e) {
                if (e.message === 'no_connect')
                    return '';
                throw new Error(vscode_api_2.I18n.common_deploy_easy.not_system_info);
            }
            return deployPackage;
        });
    }
    /**
     * 在调起终端前做一些准备工作
     * @param hasUploadFile 是否上传了安装包文件
     */
    beforeTerminal(hasUploadFile) {
        return __awaiter(this, void 0, void 0, function* () {
            // 没上传安装包则需要走联网下载流程
            if (!hasUploadFile) {
                let deployPackage = yield this.getDeployPackage();
                try {
                    yield this.uploadThroughInternet(deployPackage);
                    vscode_api_1.default.showMsg(vscode_api_2.I18n.common_tip.tool_uploaded_successfully);
                }
                catch (e) {
                    throw e;
                }
            }
        });
    }
    // 运行部署流程： 执行部署脚本，轮询部署结果
    gotoDeploy(hasUploadFile) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield this.beforeTerminal(hasUploadFile);
            }
            catch (e) {
                throw e;
            }
            // 上传一键部署脚本
            const localPath = path_1.join(vscode_api_1.default.extensionPath, this.sourceDir, this.shellName);
            const remotePath = this.workDir + this.shellName;
            try {
                const sftp = new ssh2tools_1.Sftp(this.conn);
                yield sftp.uploadFile({
                    localPath,
                    remotePath,
                    options: {
                        beforeFastPut: (uploadObj) => {
                            uploadObj.options = {
                                mode: 0o544
                            };
                        }
                    }
                });
                vscode_api_1.default.writeLog('info', '脚本上传成功');
            }
            catch (e) {
                throw e;
            }
            // 上传结束，用户交互移交终端，为响应用户操作，重新点亮下一步按钮
            this.panel.postMsg({
                command: 'upload-finished'
            });
            //  连接
            // this.connectWithTerminal();
            // 运行脚本
            this.runInstall();
            // 等待部署
            const polling = new polling_util_1.Polling();
            return new Promise((resolve, reject) => {
                polling.getResponseInterval({
                    // 轮询结果处理
                    result: {
                        success: () => {
                            this.deploySucceed();
                            resolve(true);
                        },
                        fail: () => {
                            this.deployFail();
                            resolve(false);
                        }
                    },
                    // 轮询设置，
                    // 包括轮询的核心操作checkFunc定义如何轮询
                    // success定义轮询成功的标志
                    // fail定义轮询失败的标志
                    // timeout定义轮询超时的标志 设置为0或不设置表示无超时限制
                    checkOptions: {
                        checkFunc: () => __awaiter(this, void 0, void 0, function* () {
                            let res = '';
                            try {
                                yield this.conn.exeCMDRemote(`cat ${this.logPath}`, {
                                    onData: (str) => {
                                        res = str;
                                    }
                                });
                            }
                            catch (e) {
                                throw e;
                            }
                            return res;
                        }),
                        success: (str) => {
                            vscode_api_1.default.writeLog('info', '轮询日志文件结果：' + str);
                            return /success/.test(str);
                        },
                        fail: (str) => {
                            vscode_api_1.default.writeLog('info', '轮询日志文件结果：' + str);
                            return /fail/.test(str);
                        }
                    },
                    // 轮询周期
                    interval: 2 * global_constants_1.SECOND
                });
            });
        });
    }
    // 部署成功
    deploySucceed() {
        // 将模板替换为设置密码的命令
        const tmp2RealCmd = (cmdTmp) => cmdTmp.replace(/x2openEuler_password/g, this.pwd);
        // 设置x2openEuler用户密码
        this.conn.exeCMDRemote(tmp2RealCmd(global_constants_1.SET_PWD_TMP));
        vscode_api_1.default
            .destroyOutput(this.output)
            .writeLog('info', '一键部署成功')
            .userLog('一键部署成功');
    }
    // 部署失败
    deployFail() {
        return __awaiter(this, void 0, void 0, function* () {
            // 查询包名
            try {
                const packExistName = yield this.x2openEulerPackName();
                vscode_api_1.default.userLog('一键部署失败');
                if (packExistName) {
                    // 卸载已装的包
                    yield this.conn.exeCMDRemote(`${global_constants_1.UNINTALL_PACKAGE}${packExistName}`);
                    vscode_api_1.default.writeLog('info', '一键部署失败，卸载安装包');
                }
            }
            catch (e) {
                throw e;
            }
        });
    }
    toolExists() {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const exist = yield this.x2openEulerExists();
                if (exist) {
                    vscode_api_1.default.writeLog('info', '后台已经安装工具' + exist);
                    return 'have';
                }
                vscode_api_1.default.writeLog('info', '后台没有工具，准备上传一键部署脚本');
                return 'nohave';
            }
            catch (e) {
                throw e;
            }
        });
    }
    // 部署前
    beforeDeploy() {
        return __awaiter(this, void 0, void 0, function* () {
            vscode_api_1.default.writeLog('info', '部署前期准备');
            // 构建工作目录
            return util_1.createWorkDir(this.conn, this.workDir);
        });
    }
    // 部署后清除脚本和日志文件
    afterDeploy() {
        return __awaiter(this, void 0, void 0, function* () {
            vscode_api_1.default.writeLog('info', '部署结束，清除脚本和日志文件');
            // 清除工作目录
            try {
                this.conn.rmFileRemote(this.workDir);
                vscode_api_1.default.userLog(`删除文件夹${this.workDir}`);
            }
            catch (e) {
                vscode_api_1.default.writeLog('debug', '清除临时//tmp//x2openEuler//失败');
            }
        });
    }
    // 获取进行部署的服务器连接信息
    getServerData() {
        const serverData = this.conn.getServerInfo();
        return serverData;
    }
}
exports.DeployEasy = DeployEasy;
//# sourceMappingURL=deploy_easy.js.map