"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HistoryReport = void 0;
const path_1 = require("path");
const vscode_api_1 = require("../vscode_api");
const global_constants_1 = require("../consts/global_constants");
const file_util_1 = require("../utils/file_util");
const util_1 = require("./util");
const base_util_1 = require("../utils/base_util");
const fs = require("fs");
class HistoryReport {
    constructor(type) {
        this._type = type;
        this.reportList = [];
    }
    /**
     * 获取单个类型的历史报告
     * @param userPath 用户路径
     * @param eath     报告类型
     * @returns Array
     */
    getReports(server) {
        return __awaiter(this, void 0, void 0, function* () {
            const { host, port, username } = server;
            const userPath = path_1.join(vscode_api_1.default.extensionPath, base_util_1.toSha256(`${host}${port}${username}`));
            const dirPath = path_1.join(userPath, this._type);
            const installedPath = path_1.join(userPath, 'installed');
            this.reportList = yield this.readReport(dirPath, this._type);
            let installedList;
            if (this._type == 'software') {
                installedList = yield this.readReport(installedPath, 'installed');
                installedList.map(report => {
                    this.reportList.push(report);
                });
            }
            this.reportList.sort((previous, next) => next.date - previous.date);
        });
    }
    // 读取本地历史报告
    readReport(localpath, type) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                yield file_util_1.mkDirAsync(localpath, { recursive: true });
                const files = yield file_util_1.readDirAsync(localpath);
                const paths = [];
                const details = yield Promise.all(files.map((file) => __awaiter(this, void 0, void 0, function* () {
                    const path = path_1.join(localpath, file);
                    if (type == 'software') {
                        const reportFiles = yield file_util_1.readDirAsync(path);
                        paths.push({ path: reportFiles, scanType: reportFiles.length == 2 ? 'single' : 'batch', dir: path });
                    }
                    else if (type == 'installed') {
                        const reportFiles = yield file_util_1.readDirAsync(path);
                        paths.push({ htmlPath: path_1.join(path, file + '.tar.gz'), excelPath: path_1.join(path, file + '.xls'), scanType: 'installed', dir: path });
                    }
                    else {
                        paths.push(path);
                    }
                    return file_util_1.statAsync(path);
                })));
                if (type == 'software') {
                    paths.map(item => {
                        item.path.map((file) => {
                            let _arr = file.split('.');
                            let suffix = _arr[_arr.length - 1];
                            if (suffix == 'html' && item.scanType == 'single') {
                                item.htmlPath = path_1.join(item.dir, file);
                            }
                            else if (suffix == 'gz' && item.scanType == 'batch') {
                                item.htmlPath = path_1.join(item.dir, file);
                            }
                            else if (suffix == 'xls') {
                                item.excelPath = path_1.join(item.dir, file);
                            }
                        });
                    });
                }
                let list = details.map((e, i) => {
                    const date = new Date(e.ctime);
                    return {
                        path: this._type == 'software' ? paths[i].htmlPath : paths[i],
                        excelPath: this._type == 'software' ? paths[i].excelPath : '',
                        scanType: this._type == 'software' ? paths[i].scanType : '',
                        dir: this._type == 'software' ? paths[i].dir : '',
                        name: files[i],
                        date,
                        num: i + 1
                    };
                }).sort((a, b) => b.date - a.date);
                return list;
            }
            catch (e) {
                throw e;
            }
        });
    }
    /**
     * 删除文件夹
     * @param p 文件夹路径
     */
    deleteDir(p) {
        const list = fs.readdirSync(p);
        list.forEach(v => {
            const url = p + '/' + v;
            // 读取文件信息
            const stats = fs.statSync(url);
            // 判断是文件还是文件夹
            if (stats.isFile()) {
                fs.unlinkSync(url);
            }
            else {
                this.deleteDir(url);
            }
        });
        // 删除空文件夹
        fs.rmdirSync(p);
    }
    /**
     * 删除历史报告文件
     * @param data 报告路径和序号
     */
    removeHistoryReport(data) {
        return __awaiter(this, void 0, void 0, function* () {
            vscode_api_1.default.writeLog('info', '准备删除' + data.deletePath);
            this.removeReport(data.name);
            try {
                if (this._type == 'software') {
                    this.deleteDir(data.deletePath);
                }
                else {
                    yield file_util_1.unLinkAsync(data.deletePath);
                }
                vscode_api_1.default
                    .writeLog('info', '成功删除' + data.deletePath)
                    .userLog(`成功删除历史报告：${data.deletePath}`);
            }
            catch (e) {
                vscode_api_1.default.writeLog('error', '删除失败，原因：' + e.message);
                throw e;
            }
        });
    }
    /**
     * 删除历史报告记录
     * @param type 报告类型
     * @param name  报告名称
     * @returns boolean
     */
    removeReport(name) {
        const reportIdx = this.reportList.findIndex((e) => e.name === name);
        if (reportIdx !== -1) {
            this.reportList.splice(reportIdx, 1);
            return 0;
        }
        else {
            return 1;
        }
    }
    /**
     * 查找报告
     * @param num  报告序号
     * @returns boolean
     */
    findReport(num) {
        const report = this.reportList.find((e) => e.num === num);
        return report;
    }
    /**
     * 添加历史报告
     * @param type 报告类型
     * @param report  报告数据
     * @returns boolean
     */
    pushReport(report) {
        const len = this.reportList.length;
        if (len >= global_constants_1.HISTORY_MAX_SIZE)
            return 1;
        // 将报告的num标识映射到1-HISTORY_MAX_SIZE之间
        let num = !len ? 1 : util_1.hashNum(this.reportList[len - 1].num) + 1;
        while (this.findReport(num)) {
            num = util_1.hashNum(num + 1);
        }
        this.reportList.unshift({ path: report.path, num, date: report.date, name: report.name, excelPath: report.excelPath, dir: report.dir, scanType: report.scanType });
        return 0;
    }
}
exports.HistoryReport = HistoryReport;
//# sourceMappingURL=history_report.js.map