"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.handleFileName = exports.isErrorFile = exports.checkFileType = exports.handleUpload = exports.getBaseCMD = void 0;
const vscode_api_1 = require("../vscode_api");
const file_util_1 = require("../utils/file_util");
const base_util_1 = require("../utils/base_util");
const global_constants_1 = require("../consts/global_constants");
const vscode_api_2 = require("../vscode_api/vscode_api");
const ssh2tools_1 = require("../ssh2tool/ssh2tools");
function getBaseCMD(serverConn, cmd = global_constants_1.ALIAS_TOOL) {
    return new Promise((resolve, reject) => {
        let res = '';
        let cmdResStr = '';
        serverConn.exeCMDRemote(cmd, {
            onData: (data) => {
                const reg = data.match(global_constants_1.REG_EXPS.aliasTool);
                cmdResStr += data;
                if (reg) {
                    res = reg.groups.cmd.trim();
                }
            }
        }).then(() => {
            if (res) {
                resolve(res);
            }
            else {
                reject(new Error(cmdResStr));
            }
        }).catch((e) => {
            if (e.message === 'no_connect')
                return;
            reject(e);
        });
    });
}
exports.getBaseCMD = getBaseCMD;
/**
 * 文件上传流程
 * @param isDir 是否文件夹
 * @param fileObj 文件信息
 * @param remotePath 远端路径
 * @param p vscode进度条进度记录对象
 * @param t vscode进度条取消对象
 */
function handleUpload(conn, isDir, fileObj, remotePath, p, t) {
    return __awaiter(this, void 0, void 0, function* () {
        vscode_api_1.default.writeLog('info', '正在上传文件');
        const sftp = new ssh2tools_1.Sftp(conn);
        try {
            isDir
                ? yield sftp.pureUploadDir(fileObj.path, remotePath, p, t)
                : yield sftp.pureUploadFile(fileObj.path, remotePath, p, t);
            vscode_api_1.default.showMsg(vscode_api_2.I18n.common_tip.uploaded_successfully);
            vscode_api_1.default.userLog(`上传文件${fileObj.name}成功`);
        }
        catch (e) {
            if (e !== 2 /* CANCEL */) {
                vscode_api_1.default.writeLog('error', '上传文件失败, 原因：' + e.message);
            }
            else {
                vscode_api_1.default.writeLog('info', '取消上传');
            }
            throw e;
        }
    });
}
exports.handleUpload = handleUpload;
// 检测文件是否符合后台工具包
function backendToolCheck(fileObj) {
    return base_util_1.inWhiteList(fileObj.path, file_util_1.getWhiteList('backendTool'));
}
// 检测文件是否符合单应用
function singleAppCheck(fileObj) {
    return __awaiter(this, void 0, void 0, function* () {
        if (fileObj.files) {
            // return await checkDir(fileObj.files, fileObj.path, _singleAppWhiteList);
            return true;
        }
        else {
            return file_util_1.checkFile(fileObj.path, file_util_1.getWhiteList('singleApp'));
        }
    });
}
// 检测文件是否符合批量bin
function binCheck(fileObj) {
    return __awaiter(this, void 0, void 0, function* () {
        if (fileObj.files) {
            return file_util_1.checkDir(fileObj.files, fileObj.path, file_util_1.getWhiteList('bin'), false);
        }
        else {
            return false;
        }
    });
}
// 检测文件是否符合配置导入
function confCheck(fileObj) {
    return base_util_1.inWhiteList(fileObj.path, file_util_1.getWhiteList('conf'));
}
// 检测文件是否符合硬件导入
function hardCheck(fileObj) {
    return base_util_1.inWhiteList(fileObj.path, file_util_1.getWhiteList('hard'));
}
// 检测文件类型
exports.checkFileType = {
    backendTool: backendToolCheck,
    single: singleAppCheck,
    batch: binCheck,
    conf: confCheck,
    hard: hardCheck
};
function showErrorMsg(msg, type) {
    let errorMsg = '';
    const errorHandler = {
        TypeError: () => {
            switch (type) {
                case 'single':
                    errorMsg = vscode_api_2.I18n.common_file_error.soft_single_support;
                    break;
                case 'batch':
                    errorMsg = vscode_api_2.I18n.common_file_error.soft_multi_support;
                    break;
                case 'conf':
                case 'hard':
                    errorMsg = vscode_api_2.I18n.common_file_error.conf_hard_support;
                    break;
                case 'backendTool':
                    errorMsg = vscode_api_2.I18n.common_file_error.backend_tool_support;
                    break;
                default:
                    vscode_api_1.default.writeLog('error', `无${type}文件类型`);
                    break;
            }
        },
        SizeError: () => {
            errorMsg = vscode_api_2.I18n.common_file_error.upload_size_error;
        },
        EmptyError: () => {
            errorMsg = vscode_api_2.I18n.common_file_error.empty_dir;
        },
    };
    errorHandler[msg]();
    // 延迟1s，等进度条消失再显示错误消息
    setTimeout(() => {
        vscode_api_1.default.errMsg(errorMsg);
    }, 1 * global_constants_1.SECOND);
    throw new Error(errorMsg);
}
function isErrorFile(check, fileObj, type, namesake) {
    const TE = 'TypeError';
    const SE = 'SizeError';
    const EE = 'EmptyError';
    try {
        if (!check) {
            showErrorMsg(TE, type);
        }
        if (fileObj.size >= global_constants_1.MAX_FILE_SIZE) {
            showErrorMsg(SE);
        }
        if (fileObj.size === 0) {
            showErrorMsg(EE);
        }
        if (namesake) {
            throw new Error('Namesake Error');
        }
    }
    catch (e) {
        throw e;
    }
    return 0;
}
exports.isErrorFile = isErrorFile;
/**
 * 对文件名的空格进行处理
 * @param fileName 文件名
 * @returns 处理后的文件名
 */
function handleFileName(fileName) {
    const reg = /\s+/g;
    const res = fileName.replace(reg, '_');
    return res;
}
exports.handleFileName = handleFileName;
//# sourceMappingURL=util.js.map