"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Sftp = exports.SSH2Client = void 0;
const ssh2 = require("ssh2");
const fs = require("fs");
const compressing = require("compressing");
const path_1 = require("path");
const vscode_api_1 = require("../vscode_api");
const base_util_1 = require("../utils/base_util");
const file_util_1 = require("../utils/file_util");
const base_util_2 = require("../utils/base_util");
const vscode_api_2 = require("../vscode_api/vscode_api");
const global_constants_1 = require("../consts/global_constants");
const Subscribe_1 = require("../utils/Subscribe");
let ChannelOpenFailure = '(SSH) Channel open failure: open failed';
/**
 * 管理ssh2客户端连接及方法的类,每个实例都可以创建一个ssh2连接
 * @privateVars ssh2Executer 用于创建ssh2连接、执行ssh2方法的句柄
 * @privateVars isConnected 当前连接状态
 * @privateVars server 服务器配置信息
 * @publicMethod setServerConfig(options:{server, readyTimeout, authorization})
 */
class SSH2Client {
    constructor(client) {
        // 如果参数是serverMark则找到serverMark对应的实例
        if (typeof client === 'string') {
            const instance = SSH2Client.getConnByMark(client);
            if (instance) {
                return instance;
            }
        }
        // 如果参数是连接实例，则返回这个连接
        if (client instanceof SSH2Client) {
            return client;
        }
        this._ssh2Executer = new ssh2.Client();
        this._isConnected = false;
        this.isConnectedStatus = false;
        this._server = {};
    }
    static getConnByMark(serverMark) {
        if (!serverMark)
            return null;
        const _serverMark = serverMark.trim();
        const conn = this.connections.find((e) => e.serverMark === _serverMark);
        return conn ? conn.instance : null;
    }
    static addConn(conn) {
        this.connections.push(conn);
        return this;
    }
    static removeConn(client) {
        const clientIndex = this.connections.findIndex((e) => e.instance === client);
        if (clientIndex !== -1) {
            this.connections.splice(clientIndex, 1);
        }
        return this;
    }
    /**
     * 添加对stream流上数据的监听
     * @param eventHooks 流数据监听钩子
     * @param stream 流
     */
    static _onStream(eventHooks, stream) {
        stream
            .on('data', (data) => {
            const _data = base_util_1.buf2String(data);
            base_util_1.callHooks('onData', eventHooks, _data);
        })
            .on('close', (data) => {
            const _data = base_util_1.buf2String(data);
            base_util_1.callHooks('onClose', eventHooks, _data);
        })
            .stderr.on('data', (data) => {
            const _data = base_util_1.buf2String(data);
            base_util_1.callHooks('onErrData', eventHooks, _data);
        });
    }
    // --------以上是静态成员与方法-----------
    // ---------ssh2与服务器连接相关-----------
    /**
     * 获取连接的host
     * @returns host
     */
    getServerInfo() {
        return this._server.host
            ? {
                host: this._server.host,
                port: this._server.port,
                username: this._server.username
            }
            : null;
    }
    /**
     * 获取具有root权限的用户
     * @returns root权限用户列表
     */
    getRootUsers() {
        return __awaiter(this, void 0, void 0, function* () {
            return ['root'];
        });
    }
    /**
     * 判断是否已经连接到服务器
     * @returns boolean
     */
    hasConnected2Server() {
        return this._isConnected;
    }
    /**
     * 设置需要连接的服务器的配置信息，
     * @param options readyTimeout，server，authorazition
     * @returns this
     */
    setServerConfig(options) {
        const { server, readyTimeout, authorization } = options;
        if (!server)
            return this;
        if (readyTimeout) {
            server.readyTimeout = readyTimeout;
        }
        if (authorization) {
            server.debug = (data) => {
                const debugStr = base_util_1.buf2String(data);
                if (debugStr.search(/password auth failed/) !== -1) {
                    base_util_1.callHooks('onAuthFail', authorization);
                }
                else if (debugStr.search(/ECONNREFUSED/) !== -1) {
                    base_util_1.callHooks('onConnError', authorization);
                }
                else if (debugStr.search(/ETIMEDOUT/) !== -1) {
                    base_util_1.callHooks('onConnTimeout', authorization);
                }
            };
        }
        this._server = Object.assign({}, server);
        Object.defineProperty(this._server, 'serverMark', {
            get() {
                return `${this.host}:${this.port}`;
            }
        });
        return this;
    }
    /**
     * 通过ssh2连接到服务器
     * @param eventHooks 连接后钩子
     * @returns this
     */
    connect2Server(eventHooks, restart = false) {
        this.password = this._server.password;
        if (this.hasConnected2Server() && !restart) {
            this.disconnect();
            this._ssh2Executer = new ssh2.Client();
        }
        if (restart) {
            this._ssh2Executer = new ssh2.Client();
        }
        else {
            this.eventHooks = eventHooks;
        }
        this._ssh2Executer
            .on('ready', () => {
            this._server.password = '';
            delete this._server.password;
            this._setStateConnect();
            if (restart) {
                base_util_1.callHooks('onReadyRestart', eventHooks);
            }
            else {
                base_util_1.callHooks('onReady', eventHooks);
            }
            SSH2Client.addConn({
                serverMark: this._server.serverMark,
                instance: this
            });
        })
            .on('error', (error) => {
            // app.errMsg('ssh2连接:' + JSON.stringify(error));
            const e = base_util_2.setObjectPropertyEnumerable(error);
            if (e.message) {
                // app.errMsg(e.message);
                // 打开 ssh 通道失败，重启服务器
                if (e.message === ChannelOpenFailure) {
                    this.restartConnect();
                }
            }
            vscode_api_1.default.writeLog('error', 'ssh2连接出错:' + JSON.stringify(e));
            base_util_1.callHooks('onError', eventHooks, e);
        })
            .on('close', () => {
            // app.showMsg('服务器连接已关闭！');
            vscode_api_1.default.writeLog('info', '服务器连接已关闭！');
            this._setStateDisconnect();
            SSH2Client.removeConn(this);
            base_util_1.callHooks('onClose', eventHooks);
        })
            .on('end', () => {
            base_util_1.callHooks('onEnd', eventHooks);
        })
            .on('timeout', () => {
            console.log('timeout');
        })
            .connect(this._server);
        return this;
    }
    /**
     * 断开当前ssh2连接
     * @returns this
     */
    disconnect() {
        this._ssh2Executer.end();
        this._setStateDisconnect();
        return this;
    }
    /**
     * 将连接状态设置为未连接
     * @returns this
     */
    _setStateDisconnect() {
        this._isConnected = false;
        this.isConnectedStatus = false;
        return this;
    }
    /**
     * 将连接状态设置为已连接
     * @returns this
     */
    _setStateConnect() {
        this._isConnected = true;
        this.isConnectedStatus = true;
        return this;
    }
    // ---------ssh2与服务器连接相关-----------
    // ---------ssh2命令相关---------------
    /**
     * 远程终端执行指定命令(只传第一个参数会返回一个Promise，最后获得close事件的入参)
     * @param command 指令
     * @param eventHooks 执行指令过程中事件钩子
     * @returns this
     */
    exeCMDRemote(command, eventHooks) {
        if (!this._isConnected) {
            vscode_api_1.default.errMsg(vscode_api_2.I18n.common_server_error.no_connect);
            return Promise.reject(new Error('no_connect'));
        }
        const _command = command.trim();
        return new Promise((resolve, reject) => {
            this._ssh2Executer.exec(_command, (err, stream) => {
                if (err) {
                    vscode_api_1.default.writeLog('error', 'ssh2-exeCMDRemote' + JSON.stringify(base_util_2.setObjectPropertyEnumerable(err)));
                    throw err;
                }
                SSH2Client._onStream(eventHooks, stream);
                stream.on('close', () => {
                    resolve(0 /* SUCCESS */);
                });
            });
        });
    }
    /**
     * 发送命令行移除远端资源
     * @param remotePath 远端资源路径
     * @returns PROCESS_STATUS
     */
    rmFileRemote(remotePath) {
        return this.exeCMDRemote(global_constants_1.RM + remotePath);
    }
    /**
     * 异步远端发送命令行创建文件夹
     * @param remotePath 远端文件夹位置
     * @returns PROCESS_STATUS
     */
    mkDirRemote(remotePath) {
        return __awaiter(this, void 0, void 0, function* () {
            // 移除服务器上的该文件夹,然后创建该文件夹
            yield this.rmFileRemote(remotePath);
            return this.exeCMDRemote(global_constants_1.MK_DIR + remotePath);
        });
    }
    /**
     * sftp获取失败，服务器连接断开，重新连接服务器
     */
    restartConnect() {
        vscode_api_1.default.showMsg('服务重新连接中，请稍等...');
        vscode_api_1.default.writeLog('info', '服务重新连接中...');
        if (this.setTime)
            clearTimeout(this.setTime);
        this.setTime = setTimeout(() => {
            this.setTime = undefined;
            this._server.password = this.password;
            this.connect2Server(Object.assign(Object.assign({}, this.eventHooks), { onReadyRestart: () => {
                    vscode_api_1.default.writeLog('info', '服务重新连接成功');
                    vscode_api_1.default.showMsg('服务重新连接成功!');
                    Subscribe_1.default.emit('restartConnectSuccess');
                } }), true);
        }, 5000);
    }
    /**
     * 获取sftp对象
     * @param callback 用来接收sftp句柄的回调
     * @returns this
     */
    sftp(callback) {
        if (!this._isConnected) {
            throw new Error(vscode_api_2.I18n.common_server_error.no_connect);
        }
        this._ssh2Executer.sftp((err, sftp) => {
            Subscribe_1.default.off('restartConnectSuccess');
            if (err) {
                // app.errMsg('sftp对象获取失败:' + JSON.stringify(err));
                const e = base_util_2.setObjectPropertyEnumerable(err);
                vscode_api_1.default.writeLog('error', 'sftp对象获取失败:' + JSON.stringify(e));
                vscode_api_1.default.writeLog('error', 'sftp:' + JSON.stringify(sftp));
                if (e.message === ChannelOpenFailure) {
                    this.restartConnect();
                    Subscribe_1.default.on('restartConnectSuccess', () => {
                        Subscribe_1.default.off('restartConnectSuccess');
                        this.sftp(callback);
                    });
                }
                else {
                    throw err;
                }
            }
            if (sftp)
                callback.call(this, sftp);
        });
        return this;
    }
    /**
     * 查看信息
     * @param command 命令
     * @returns string
     */
    checkInfo(command) {
        return __awaiter(this, void 0, void 0, function* () {
            let data;
            try {
                yield this.exeCMDRemote(command, {
                    onData: (res) => {
                        data = res.trim();
                    }
                });
            }
            catch (e) {
                if (e.message === 'no_connect') {
                    throw e;
                }
                throw new Error(vscode_api_2.I18n.common_deploy_easy.not_system_info);
            }
            return data;
        });
    }
}
exports.SSH2Client = SSH2Client;
// --------静态成员与方法-----------
SSH2Client.connections = [];
/**
 * sftp类
 * 有两种构造方式，传一个ssh2对象或者不传
 * 传参则sftp对象使用传的参数的ssh连接，不传则需要构造新的连接
 */
class Sftp {
    constructor(conn) {
        this._conn = conn;
        this._cancelSftp = false;
        this._currentSftp = null;
    }
    /**
     * 取消sftp传输
     * @returns this
     */
    cancelSftp() {
        if (!this._currentSftp)
            return this;
        this._cancelSftp = true;
        this._currentSftp.end();
        this._currentSftp = null;
        return this;
    }
    /**
     * 修改远端文件的权限
     * @param remotePath 远端文件路径
     * @param mode 远端文件权限模式
     * @returns PROCESS_STATUS
     */
    changeRemoteFileMod(remotePath, mode) {
        const _remotePath = remotePath.trim();
        return new Promise((resolve, reject) => {
            this._conn.sftp((sftp) => {
                sftp.chmod(_remotePath, mode, (err) => {
                    if (err) {
                        throw err;
                    }
                    resolve(0 /* SUCCESS */);
                    sftp.end();
                });
            });
        });
    }
    /**
     * 获取一个写远程文件的流
     * @param remotePath 远程路径
     * @param options 配置对象
     * @returns 流
     */
    createWriteStream(remotePath, options) {
        const _remotePath = remotePath;
        let res;
        this._conn.sftp((sftp) => {
            res = sftp.createWriteStream(_remotePath, options);
        });
        return res;
    }
    /**
     * 判断远端服务器上是否存在上传的安装包
     * @param path 文件路径
     * @returns PROCESS_STATUS
     */
    packExists(path) {
        const _path = path.trim();
        return new Promise((resolve, reject) => {
            this._conn.sftp((sftp) => __awaiter(this, void 0, void 0, function* () {
                sftp.exists(_path, (isExist) => {
                    resolve(isExist);
                });
            }));
        });
    }
    pureUploadFile(localPath, remotePath, p, t, options = {}) {
        let last = 0;
        return this.uploadFile({
            localPath,
            remotePath,
            options: {
                beforeFastPut: (uploadObj) => {
                    uploadObj.options = Object.assign(Object.assign({}, uploadObj.options), options);
                    t.onCancellationRequested(() => {
                        this.cancelSftp();
                        this._conn.exeCMDRemote(global_constants_1.RM + remotePath);
                    });
                },
                onFastPut: (per) => {
                    const increment = per - last;
                    last = per;
                    p.report({
                        increment,
                        message: per.toFixed(2) + '%'
                    });
                }
            }
        });
    }
    pureUploadDir(localPath, remotePath, p, t) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const filesLen = yield file_util_1.getAllFiles(localPath);
                let totalProgress = 0;
                let last = 0;
                return this.uploadDir({
                    localPath,
                    remotePath,
                    options: {
                        beforeFastPut: () => {
                            t.onCancellationRequested(() => {
                                this.cancelSftp();
                                this._conn.exeCMDRemote(global_constants_1.RM + remotePath);
                            });
                        },
                        onFastPut: (per) => {
                            const increment = per - last;
                            last = per;
                            p.report({
                                increment,
                                message: per.toFixed(2) + '%'
                            });
                        }
                    }
                });
            }
            catch (e) {
                throw e;
            }
        });
    }
    /**
     * 上传一个文件
     * @param uploadOptions localPath, remotePath, uploadHooks
     * @returns PROCESS_STATUS
     */
    uploadFile(uploadOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            const _localPath = uploadOptions.localPath;
            const _remotePath = uploadOptions.remotePath;
            const _fileHooks = uploadOptions && uploadOptions.options;
            try {
                return yield this.fastPut(_localPath, _remotePath, _fileHooks);
            }
            catch (e) {
                throw e;
            }
        });
    }
    /**
     * 上传一个文件夹
     * @param uploadOptions localPath, remotePath, uploadHooks
     * @returns PROCESS_STATUS
     */
    uploadDir(uploadOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const _localPath = uploadOptions.localPath;
                const _remotePath = uploadOptions.remotePath;
                const _dirHooks = uploadOptions && uploadOptions.options;
                base_util_1.callHooks('beforeUploadDir', _dirHooks);
                const dir = _localPath.split('/').pop();
                const name = dir + '.zip';
                const extensionGZ = path_1.join(vscode_api_1.default.extensionPath, 'extension', 'ssh2tool', 'upload', name);
                yield compressing.zip.compressDir(_localPath, extensionGZ);
                const _options = createFtpFile(extensionGZ, `${_remotePath}.zip`, _dirHooks);
                yield this.uploadFile(_options);
                fs.unlinkSync(extensionGZ);
                yield this._conn.exeCMDRemote(`cd ${global_constants_1.WORK_DIR}; rm -rf ${_remotePath}; unzip ${name}; rm -f ${name}`);
                return 0 /* SUCCESS */;
            }
            catch (e) {
                vscode_api_1.default.writeLog('error', 'sftp-uploadDir' + JSON.stringify(base_util_2.setObjectPropertyEnumerable(e)));
                throw e;
            }
        });
    }
    /**
     * 下载一个文件
     * @param downloadOptions localPath, remotePath, downloadHooks
     * @returns PROCESS_STATUS
     */
    downloadFile(downloadOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            const _localPath = downloadOptions.localPath;
            const _remotePath = downloadOptions.remotePath;
            const _fileHooks = downloadOptions.options;
            try {
                return this.fastGet(_remotePath, _localPath, _fileHooks);
            }
            catch (e) {
                console.log(e);
                throw e;
            }
        });
    }
    readDir(location) {
        this._cancelSftp = false;
        return new Promise((resolve, reject) => {
            this._conn.sftp((sftp) => __awaiter(this, void 0, void 0, function* () {
                if (this._currentSftp) {
                    reject(new Error('还有sftp进程未结束'));
                }
                this._currentSftp = sftp;
                sftp.readdir(location, (err, list) => {
                    if (err) {
                        reject(err);
                    }
                    else {
                        resolve(list);
                    }
                    this._currentSftp = null;
                });
            }));
        });
    }
    /**
     * 下载一个文件夹
     */
    downloadDir(downloadOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            const _localPath = downloadOptions.localPath;
            const _remoteDir = downloadOptions.remotePath;
            const _dirHooks = downloadOptions.options;
            base_util_1.callHooks('beforeDownloadDir', _dirHooks);
            try {
                const files = yield this.readDir(_remoteDir);
                let promise = Promise.resolve(0);
                for (const fileObj of files) {
                    if (fileObj.attrs.mode == 33184) {
                        const remoteFile = `${_remoteDir}/${fileObj.filename}`;
                        const localFile = path_1.join(_localPath, fileObj.filename);
                        try {
                            // await this.downloadFile({
                            //   remotePath: remoteFile,
                            //   localPath: localFile
                            // });
                            promise = promise.then(() => {
                                return this.downloadFile({
                                    remotePath: remoteFile,
                                    localPath: localFile
                                });
                            });
                        }
                        catch (e) {
                            console.log(e);
                            throw e;
                        }
                    }
                }
                yield promise;
                return 0 /* SUCCESS */;
            }
            catch (e) {
                throw e;
            }
        });
    }
    /**
     * sftp文件上传方法封装
     * @param localPath 本地路径
     * @param remotePath 远端路径
     * @param eventHooks 事件回调
     * @returns Promise<PROCESS_STATUS>
     */
    fastPut(localPath, remotePath, eventHooks) {
        // 1.初始化sftp的取消标志
        this._cancelSftp = false;
        // 2.上传
        return new Promise((resolve, reject) => {
            this._conn.sftp((sftp) => {
                if (this._currentSftp) {
                    throw new Error('还有sftp进程未结束');
                }
                this._currentSftp = sftp;
                const _options = {
                    step: (uploaded, chunk, total) => {
                        const percent = base_util_1.percentage(uploaded, total);
                        base_util_1.callHooks('onFastPut', eventHooks, percent);
                    }
                };
                const uploadObj = createFtpFile(localPath.trim(), remotePath.trim(), _options);
                // 上传之前为用户提供一个修改的机会
                base_util_1.callHooks('beforeFastPut', eventHooks, uploadObj);
                sftp.fastPut(uploadObj.localPath, uploadObj.remotePath, uploadObj.options, (err) => {
                    Subscribe_1.default.off('restartConnectSuccess');
                    base_util_1.callHooks('afterFastPut', eventHooks, err);
                    if (this._cancelSftp) {
                        reject(2 /* CANCEL */);
                    }
                    else {
                        if (err) {
                            const e = base_util_2.setObjectPropertyEnumerable(err);
                            vscode_api_1.default.errMsg('上传文件失败:' + JSON.stringify(e));
                            vscode_api_1.default.writeLog('error', '上传文件失败' + JSON.stringify(e));
                            vscode_api_1.default.writeLog('error', '上传文件失败:' + JSON.stringify(uploadObj));
                            Subscribe_1.default.on('restartConnectSuccess', () => {
                                Subscribe_1.default.off('restartConnectSuccess');
                                return this.fastPut(localPath, remotePath, eventHooks);
                            });
                            // throw err;
                        }
                        else {
                            resolve(0 /* SUCCESS */);
                        }
                        sftp.end();
                        this._currentSftp = null;
                    }
                });
            });
        });
    }
    /**
     * sftp文件下载方法封装
     * @param remotePath 远端路径
     * @param localPath 本地路径
     * @param eventHooks 事件回调
     * @returns Promise<PROCESS_STATUS>
     */
    fastGet(remotePath, localPath, eventHooks) {
        // 1.初始化sftp的取消标志
        this._cancelSftp = false;
        // 2.下载
        return new Promise((resolve, reject) => {
            this._conn.sftp((sftp) => {
                if (this._currentSftp) {
                    vscode_api_1.default.errMsg('还有sftp进程未结束');
                    vscode_api_1.default.writeLog('error', '还有sftp进程未结束');
                    throw new Error('还有sftp进程未结束');
                }
                this._currentSftp = sftp;
                const _options = {
                    step: (uploaded, chunk, total) => {
                        const percent = base_util_1.percentage(uploaded, total);
                        base_util_1.callHooks('onFastGet', eventHooks, percent);
                    }
                };
                const downloadObj = createFtpFile(localPath.trim(), remotePath.trim(), _options);
                // 上传之前为用户提供一个修改的机会
                base_util_1.callHooks('beforeFastGet', eventHooks, downloadObj);
                sftp.fastGet(downloadObj.remotePath, downloadObj.localPath, downloadObj.options, (err) => {
                    base_util_1.callHooks('afterFastGet', eventHooks, err);
                    Subscribe_1.default.off('restartConnectSuccess');
                    if (this._cancelSftp) {
                        reject(2 /* CANCEL */);
                    }
                    else {
                        if (err) {
                            const e = base_util_2.setObjectPropertyEnumerable(err);
                            vscode_api_1.default.errMsg('下载文件失败:' + JSON.stringify(e));
                            vscode_api_1.default.writeLog('error', '下载文件失败:' + JSON.stringify(downloadObj));
                            vscode_api_1.default.writeLog('error', JSON.stringify(e));
                            Subscribe_1.default.on('restartConnectSuccess', () => {
                                Subscribe_1.default.off('restartConnectSuccess');
                                return this.fastGet(remotePath, localPath, eventHooks);
                            });
                            // throw err;
                        }
                        else {
                            resolve(0 /* SUCCESS */);
                        }
                    }
                    this._currentSftp = null;
                    sftp.end();
                });
            });
        });
    }
}
exports.Sftp = Sftp;
/**
 * 构造一个sftp方法所需要使用的对象
 * @param localPath 本地路径
 * @param remotePath 远端路径
 * @param options 设置对象
 * @returns 三个参数所构成的对象
 */
function createFtpFile(localPath, remotePath, options) {
    return {
        localPath,
        remotePath,
        options
    };
}
//# sourceMappingURL=ssh2tools.js.map