"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getIconPath = exports.createVoidServerNode = exports.createServerNode = exports.createReportNode = exports.createRootNode = exports.createVoidNode = exports.getServerItems = exports.getReportItems = exports.getRootItems = void 0;
const vscode = require("vscode");
const vscode_api_1 = require("../vscode_api");
const vscode_api_2 = require("../vscode_api/vscode_api");
const treedata_config_1 = require("./treedata.config");
const base_util_1 = require("../utils/base_util");
const path_1 = require("path");
const global_constants_1 = require("../consts/global_constants");
const server_manage_1 = require("../server_manage");
const server_estimate_1 = require("../server_estimate");
const crypto_1 = require("../crypto");
const ssh2tool_1 = require("../ssh2tool");
/**
 * 返回树的根节点数据
 * @returns Array<TreeNode>
 */
function getRootItems() {
    if (Object.keys(server_manage_1.serverManage.active).length === 0) {
        return [treedata_config_1.rootItems[0]].map((item) => createRootNode(item));
    }
    else {
        return treedata_config_1.rootItems.map((item) => createRootNode(item));
    }
}
exports.getRootItems = getRootItems;
/**
 * 根据报告类型去返回树的报告列表节点数据
 * @param reportType 报告的类型
 * @returns Array<TreeNode>
 */
function getReportItems(reportType) {
    let conn = new ssh2tool_1.default.SSH2Client(server_manage_1.serverManage.active.serverMark);
    let list;
    switch (reportType) {
        case 'software':
            list = new server_estimate_1.SoftwareEstimate(conn).historyReport.reportList;
            break;
        case 'config':
            list = new server_estimate_1.DeployEstimate(conn).historyReport.reportList;
            break;
        case 'hardware':
            list = new server_estimate_1.HardwareEstimate(conn).historyReport.reportList;
            break;
    }
    return list.map((item) => createReportNode(item, reportType));
}
exports.getReportItems = getReportItems;
/**
 * 返回树的服务器列表节点数据
 * @returns Array<TreeNode>
 */
function getServerItems() {
    // 从服务器管理获取服务器信息
    const serverList = server_manage_1.serverManage.serverArray;
    const active = server_manage_1.serverManage.active;
    if (serverList.length) {
        return serverList.map((item) => {
            if (active.serverMark === item.serverMark) {
                item.active = true;
            }
            else {
                item.active = false;
            }
            return createServerNode(item);
        });
    }
    else {
        return [createVoidServerNode()];
    }
}
exports.getServerItems = getServerItems;
/**
 * 构造一个TreeNode类型的空节点
 * @returns TreeNode
 */
function createVoidNode(item) {
    let treeNode;
    treeNode = {
        label: vscode_api_2.I18n.common_tip.no_report,
        collapsibleState: vscode.TreeItemCollapsibleState.None,
        contextValue: "zero" /* VOID */,
        nodeType: "zero" /* VOID */
    };
    return treeNode;
}
exports.createVoidNode = createVoidNode;
/**
 * 构造一个TreeNode类型的根节点
 * @returns TreeNode
 */
function createRootNode(item) {
    let treeNode;
    treeNode = Object.assign({}, item);
    return treeNode;
}
exports.createRootNode = createRootNode;
/**
 * 构造一个TreeNode类型的报告节点
 * @returns TreeNode
 */
function createReportNode(item, reportType) {
    let treeNode;
    let treeType;
    switch (reportType) {
        case 'software':
            treeType = "softwareReport" /* SOFTWAREREPORT */;
            break;
        case 'config':
            treeType = "configReport" /* CONFIGREPORT */;
            break;
        case 'hardware':
            treeType = "hardwareReport" /* HARDWAREREPORT */;
            break;
    }
    treeNode = Object.assign({ contextValue: treeType, label: item.name + '\n' + base_util_1.filterDate(item.date), collapsibleState: vscode.TreeItemCollapsibleState.None, nodeType: treeType, command: {
            title: vscode_api_2.I18n.common_tip.show_report,
            tooltip: item.name,
            command: global_constants_1.EXT_COMMANDS.showReport,
            arguments: [{
                    path: item.path,
                    date: item.date,
                    name: item.name,
                    excelPath: item.excelPath,
                    scanType: item.scanType,
                    dir: item.dir
                }]
        } }, item);
    return treeNode;
}
exports.createReportNode = createReportNode;
/**
 * 构造一个TreeNode类型的服务器节点
 * @returns TreeNode
 */
function createServerNode(item) {
    let treeNode;
    let havePwd;
    if (!item.active) {
        crypto_1.getPwd(item.username, item.serverMark, (pwd) => {
            if (typeof (pwd) === 'string') {
                havePwd = true;
            }
            else {
                havePwd = false;
            }
        });
    }
    treeNode = {
        label: `${item.host}:${item.port}`,
        contextValue: item.active ? "server" /* NOTCONNECTSERVER */ : havePwd ? "connectServer" /* CONNECTSERVER */ : "server" /* NOTCONNECTSERVER */,
        collapsibleState: vscode.TreeItemCollapsibleState.None,
        nodeType: item.active ? "server" /* NOTCONNECTSERVER */ : havePwd ? "connectServer" /* CONNECTSERVER */ : "server" /* NOTCONNECTSERVER */,
        active: item.active ? `${item.host}:${item.port}` : undefined,
        info: item,
        command: {
            title: vscode_api_2.I18n.common_tip.configure_server,
            tooltip: item.host,
            command: global_constants_1.EXT_COMMANDS.editServer,
            arguments: [{
                    info: item
                }]
        },
    };
    return treeNode;
}
exports.createServerNode = createServerNode;
/**
 * 构造一个TreeNode类型的空服务器节点
 * @returns TreeNode
 */
function createVoidServerNode() {
    let treeNode;
    treeNode = {
        label: vscode_api_2.I18n.common_server_manage.no_servers,
        collapsibleState: vscode.TreeItemCollapsibleState.None,
        contextValue: "zero" /* VOID */,
        nodeType: "zero" /* VOID */,
    };
    return treeNode;
}
exports.createVoidServerNode = createVoidServerNode;
/**
 * 获取服务器节点的图标路径
 * @param label 节点名
 * @param active 激活的服务器名
 * @returns string
 */
function getIconPath(label, active) {
    const treeitemDir = path_1.join(vscode_api_1.default.extensionPath, 'images', 'treeitem');
    return label === active
        ? path_1.join(treeitemDir, 'server_active.png')
        : path_1.join(treeitemDir, 'server_default.png');
}
exports.getIconPath = getIconPath;
//# sourceMappingURL=util.js.map