"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.downloadWebFile = void 0;
const fs = require("fs");
const axios_1 = require("axios");
const global_constants_1 = require("../consts/global_constants");
const vscode_api_1 = require("../vscode_api/vscode_api");
/**
 * 初始化一个进度信息更新器
 * @param res 网络请求的响应对象，从中可以获取返回结果的总长度
 * @param fsWriteStream 文件流，从中可以获取当前写入硬盘的长度
 * @param cb 对于下载进度的处理回调，将会获得三个参数，loaded已写入硬盘字节数，total文件总字节数，speed每个interval写入字节数
 * @param interval 更新进度信息的间隔时间，单位为秒
 * @returns { 定时器id， 定时器间隔时间 }
 */
function initDownloadProgressUpdater(res, fsWriteStream, cb, interval = 1) {
    let lastWritten = 0;
    let speed = 0;
    cb(fsWriteStream.bytesWritten, Number(res.headers['content-length']), speed);
    const timer = setInterval(() => {
        const loaded = fsWriteStream.bytesWritten;
        const total = Number(res.headers['content-length']);
        speed = (loaded - lastWritten) / interval;
        lastWritten = loaded;
        cb(loaded, total, speed);
    }, interval * global_constants_1.SECOND);
    return {
        timer,
        interval
    };
}
/**
 * axios响应拦截
 */
function setInterceptor() {
    axios_1.default.interceptors.response.use((res) => {
        if (res.headers['content-type'] === 'application/x-redhat-package-manager') {
            return res;
        }
        else {
            return Promise.reject(new Error('not expected response'));
        }
    }, (error) => {
        let newErr = error;
        // 响应状态码错误
        if (error.response) {
            newErr = new Error(vscode_api_1.I18n.common_tip.server_error);
            // 请求无响应
        }
        else if (error.request) {
            newErr = new Error('net error');
        }
        return Promise.reject(newErr);
    });
}
/**
 * 下载网络文件的接口
 * @param 配置对象：{
 *   url: 资源路径
 *   dstFilePath: 下载到远端路径
 *   cancel: 取消接口
 * }
 * @returns Promise （resolve返回dstFilePath的值，reject返回错误对象）
 */
function downloadWebFile({ url = '', localPath = '', cancel = { isCancel: false, onCancel: (fn) => { } }, progress = (load, total, speed) => { } }) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!url || !localPath) {
            throw new Error(`unexcepted undefined param:${url ? '<Object>.url' : '<Object>.dstFilePath'}`);
        }
        return new Promise((resolve, reject) => {
            let progressConfig;
            const fsWriteStream = fs.createWriteStream(localPath);
            fsWriteStream
                .on('finish', () => {
                if (progressConfig) {
                    setTimeout(() => {
                        clearInterval(progressConfig.timer);
                    }, progressConfig.interval);
                }
                !cancel.isCancel
                    ? resolve(0 /* SUCCESS */)
                    : reject(new Error(vscode_api_1.I18n.common_tip.cancel_download));
            });
            fsWriteStream
                .on('error', (e) => {
                reject(e);
            });
            setInterceptor();
            cancel.isCancel = false;
            axios_1.default({
                url,
                method: 'get',
                responseType: 'stream',
                timeout: 10 * global_constants_1.SECOND
            }).then((res) => {
                progressConfig = initDownloadProgressUpdater(res, fsWriteStream, progress);
                res.data.pipe(fsWriteStream);
            }).catch((error) => {
                fsWriteStream.end();
                reject(error);
            });
            cancel.onCancel(() => {
                cancel.isCancel = true;
                fsWriteStream.end();
            });
        });
    });
}
exports.downloadWebFile = downloadWebFile;
//# sourceMappingURL=web_request.js.map