"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.progressOption = exports.I18nReplace = exports.I18n = void 0;
const vscode = require("vscode");
const log_manager_1 = require("../log_manager/log_manager");
const i18n_service_1 = require("../utils/i18n_service");
exports.I18n = i18n_service_1.I18nService.I18n();
exports.I18nReplace = i18n_service_1.I18nService.I18nReplace;
// 默认的进度条配置对象
exports.progressOption = null;
class VscodeAPI {
    constructor(extensionPath, options) {
        this.extensionPath = extensionPath;
        this.$context = vscode;
        this.language = vscode.env.language;
        this.output = [];
        this.logLevel = (options && options.logLevel) || 'debug';
        this.logger = log_manager_1.default;
        this.levelObj = { none: 0, debug: 1, info: 2, warn: 3, error: 4 };
        this.extTerminals = [];
        vscode.window.onDidChangeActiveTerminal((currentTerminal) => {
            this.currentActiveTerminal = currentTerminal;
        });
        vscode.window.onDidCloseTerminal((closeTerminal) => {
            this.removeExtensionTerminals(closeTerminal);
        });
    }
    /**
     * 保存插件路径，初始化日志工具
     * @param path 插件路径
     * @returns this
     */
    setContextPath(path) {
        this.extensionPath = path;
        this.logger.init();
        return this;
    }
    /**
     * 修改进度条的默认配置
     * @param options 配置对象
     */
    setProgressOption(options = {}) {
        exports.progressOption = {
            cancellable: false,
            location: vscode.ProgressLocation.Notification,
            title: exports.I18n.common_tip.uploading // '正在上传中：'
        };
        options.title && (exports.progressOption.title = options.title);
        options.cancellable && (exports.progressOption.cancellable = options.cancellable);
        options.location && (exports.progressOption.location = options.location);
    }
    /**
     * vscode.window.showInformationMessage的简写
     * @param rest
     * @returns this
     */
    showMsg(...rest) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.$context.window.showInformationMessage(...rest);
        });
    }
    /**
     * vscode.window.showErrorMessage的简写
     * @param rest
     * @returns
     */
    errMsg(...rest) {
        this.$context.window.showErrorMessage(...rest);
        return this;
    }
    /**
     * vscode.window.showSaveDialog的简写
     * @param rest
     * @returns vscode.Uri
     */
    showSaveDialog(options) {
        return __awaiter(this, void 0, void 0, function* () {
            const uri = yield this.$context.window.showSaveDialog(options);
            return uri;
        });
    }
    /**
     * vscode.window.showOpenDialog的简写
     * @param rest
     * @returns vscode.Uri
     */
    showOpenDialog(options) {
        return __awaiter(this, void 0, void 0, function* () {
            const uri = yield this.$context.window.showOpenDialog(options);
            return uri;
        });
    }
    /**
     * vscode.commands.registerCommand的简写
     * @param rest
     * @returns 原本的return
     */
    registerCMD(...rest) {
        return this.$context.commands.registerCommand(...rest);
    }
    /**
     * vscode.commands.executeCommand的简写
     * @param rest
     * @returns 命令回调的return
     */
    executeCMD(...rest) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.$context.commands.executeCommand(...rest);
        });
    }
    /**
     * vscode.window.createWebviewPanel的简写
     * @param rest
     * @returns 返回一个webviewPanel对象
     */
    createPanel(...rest) {
        return this.$context.window.createWebviewPanel(...rest);
    }
    /**
     * vscode.window.createOutputChannel的简写
     * @param name
     * @returns 返回一个output对象
     */
    createOutput(name) {
        const output = {
            name,
            channel: this.$context.window.createOutputChannel(name),
            visible: false,
            isAlive: true
        };
        this.output.push(output);
        return output;
    }
    /**
     * 写output
     * @param msg
     * @returns output
     */
    writeOutput(msg, level, name = 'connect', clear, show) {
        let output = this.output.find(e => e.name === name);
        if (!output) {
            output = this.createOutput(name);
        }
        output.isAlive = true;
        show && this.showOutput(output);
        if (this.levelObj[level] >= this.levelObj[this.logLevel]) {
            clear && output.channel.clear();
            output.channel.appendLine(msg);
        }
        return output;
    }
    /**
     * show a output
     * @param output
     * @returns this
     */
    showOutput(output) {
        if (!output.isAlive)
            return this;
        this.output.forEach(e => e.visible = false);
        output.visible = true;
        output.channel.show();
        return this;
    }
    /**
     * hide current output
     * @returns this
     */
    hideCurrentOutput() {
        const output = this.output
            .filter(e => e.isAlive)
            .find(e => e.visible);
        output && output.channel.hide();
        return this;
    }
    /**
     * 使一个output结束生命
     * @param output
     * @returns this
     */
    destroyOutput(output) {
        output.channel.hide();
        output.isAlive = false;
        return this;
    }
    /**
     * 写log
     * @param type log类型debug，info，warn，error
     * @param msg log文本
     * @returns
     */
    writeLog(type, msg) {
        this.logger[type](msg);
        return this;
    }
    /**
     * 写操作日志
     * @param msg
     */
    userLog(msg) {
        this.logger.logOpt(msg);
    }
    msgNlog(type, msg) {
        if (type === 'error') {
            this.errMsg(msg);
        }
        else {
            this.showMsg(msg);
        }
        this.writeLog(type, msg);
        return this;
    }
    /**
     * vscode.window.createTerminal的简写
     * @param options
     * @returns 返回一个terminal对象
     */
    createTerminal(options = {}) {
        return this.$context.window.createTerminal(options);
    }
    /**
     * 获取当前focus的终端
     * @returns terminal
     */
    getCurrentTerminal() {
        return this.currentActiveTerminal;
    }
    /**
     * 移除插件创建的终端
     */
    removeExtensionTerminals(terminal) {
        if (!this.extTerminals.includes(terminal))
            return;
        const terminalIndex = this.extTerminals.findIndex((extTerminal) => terminal === extTerminal);
        this.extTerminals.splice(terminalIndex, 1);
    }
    /**
     * 判断终端是否在由插件创建的终端列表中
     * @param terminal
     * @returns true/false
     */
    isExtensionTerminal(terminal) {
        return this.extTerminals.includes(terminal);
    }
    /**
     * 获取一个特定的终端
     * @param name 终端名
     * @returns
     */
    getTerminal(options, show = true, clear = true) {
        if (options instanceof String) {
            options = {
                name: options
            };
        }
        const terminals = this.$context.window.terminals;
        let terminal = terminals.find((t) => t.name === options.name);
        if (!terminal) {
            terminal = this.createTerminal(options);
            this.extTerminals.push(terminal);
        }
        else {
            clear && terminal.sendText('clear');
        }
        show && terminal.show();
        return terminal;
    }
    /**
     * vscode.window.registerTreeDataProvider的简写
     * @param rest
     * @returns this
     */
    registerTreeProvider(...rest) {
        return this.$context.window.registerTreeDataProvider(...rest);
    }
    /**
     * vscode.window.withProgress的简写
     * @param rest
     * @returns
     */
    showProgress(...rest) {
        exports.progressOption || this.setProgressOption();
        if (rest.length === 1) {
            const res = this.$context.window.withProgress(exports.progressOption, ...rest);
            exports.progressOption = null;
            return res;
        }
        return this.$context.window.withProgress(...rest);
    }
    catchError(e) {
        this
            .errMsg(e.message)
            .writeLog('error', e.message);
    }
}
exports.default = VscodeAPI;
//# sourceMappingURL=vscode_api.js.map