"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.panelOnLeave = exports.panelOnShow = exports.updateHistoryReportList = exports.changeActiveOutput = exports.changeActiveTerminal = exports.createVpanel = exports.asWebviewUri = exports.vscodeUri = exports.getHTML = void 0;
const vscode = require("vscode");
const path = require("path");
const vscode_api_1 = require("../vscode_api");
const file_util_1 = require("../utils/file_util");
const msg_watcher_1 = require("../msg_manager/msg_watcher");
const request_1 = require("../msg_manager/request");
const server_estimate_1 = require("../server_estimate");
const estimate_config_1 = require("../server_estimate/estimate.config");
const global_constants_1 = require("../consts/global_constants");
function getHTML() {
    return __awaiter(this, void 0, void 0, function* () {
        // 读取webview页面
        try {
            const htmlPath = path.resolve(__dirname, '../webview/index.html');
            return file_util_1.readFileAsync(htmlPath, 'utf-8');
        }
        catch (e) {
            throw e;
        }
    });
}
exports.getHTML = getHTML;
/**
 * 将路径转换为vscode可识别的文件标识
 * @param pathStr
 * @returns
 */
function vscodeUri(pathStr) {
    return vscode.Uri.file(pathStr);
}
exports.vscodeUri = vscodeUri;
/**
 * 将文件协议处理为webview可以导入的形式
 * @param panel
 * @param diskPath
 * @returns
 */
function asWebviewUri(panel, diskPath) {
    return `${panel.view.webview.asWebviewUri(diskPath)}`;
}
exports.asWebviewUri = asWebviewUri;
/**
 * 构造一个ViewPanel对象
 * @param id
 * @param panel
 * @returns
 */
function createVpanel(id, panel) {
    const msgWatcher = new msg_watcher_1.MsgWatcher(panel);
    // 包含各种通信方法的panel接口的实现
    const vpanel = {
        id,
        view: panel,
        html: '',
        postMsg: (data) => {
            panel.webview.postMessage(data);
        },
        receive: msgWatcher.subVscode.bind(msgWatcher),
        unreceive: msgWatcher.removeSub.bind(msgWatcher),
        stored: {
            serverMark: '',
            conn: null,
            deploy: null,
            output: null,
            evaluationStatus: ''
        }
    };
    // 请求类的实例
    const vxios = new request_1.Vxios(vpanel);
    vpanel.request = vxios.request.bind(vxios);
    return vpanel;
}
exports.createVpanel = createVpanel;
function changeActiveTerminal(panel) {
    const activeTerminal = vscode_api_1.default.getCurrentTerminal();
    if (vscode_api_1.default.isExtensionTerminal(activeTerminal)) {
        const terminal = panel.stored.deploy && panel.stored.deploy.getDeployTerminal();
        if (terminal) {
            terminal.show();
        }
        else {
            activeTerminal.hide();
        }
    }
}
exports.changeActiveTerminal = changeActiveTerminal;
function changeActiveOutput(panel) {
    const output = panel.stored.output;
    if (output && output.isAlive) {
        vscode_api_1.default.showOutput(output);
    }
}
exports.changeActiveOutput = changeActiveOutput;
function updateHistoryReportList(panel, conn, page) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const serverInfo = conn.getServerInfo();
            // 当前页面是root用户连接的话不用更新历史报告
            if (serverInfo.username === 'root')
                return;
            vscode_api_1.default.writeLog('info', '准备获取本地历史报告');
            yield new server_estimate_1.SoftwareEstimate(conn).historyReport.getReports(conn.getServerInfo());
            yield new server_estimate_1.DeployEstimate(conn).historyReport.getReports(conn.getServerInfo());
            yield new server_estimate_1.HardwareEstimate(conn).historyReport.getReports(conn.getServerInfo());
            const arr = ['software', 'config', 'hardware'];
            let list;
            const deleteWare = arr.filter((item) => page.indexOf(item) > 0)[0];
            switch (deleteWare) {
                case 'software':
                    list = new server_estimate_1.SoftwareEstimate(conn).historyReport.reportList;
                    break;
                case 'config':
                    list = new server_estimate_1.DeployEstimate(conn).historyReport.reportList;
                    break;
                case 'hardware':
                    list = new server_estimate_1.HardwareEstimate(conn).historyReport.reportList;
                    break;
                default:
                    vscode_api_1.default.writeLog('error', '获取报告失败');
                    break;
            }
            vscode_api_1.default.writeLog('info', '获取本地历史报告成功');
            panel.postMsg({
                command: 'update-history',
                data: {
                    msg: list,
                    Introduction: estimate_config_1.server
                }
            });
        }
        catch (e) {
            throw e;
        }
    });
}
exports.updateHistoryReportList = updateHistoryReportList;
/**
 * 页面显示到顶部时的回调
 * @param panel 页面
 * @param page 页面名字
 * @returns void
 */
function panelOnShow(panel, page) {
    if (!panel.view.visible)
        return;
    changeActiveOutput(panel);
    const conn = panel.stored.conn;
    if (!conn || !conn.hasConnected2Server())
        return;
    updateHistoryReportList(panel, conn, page);
}
exports.panelOnShow = panelOnShow;
/**
 * 页面关闭时的回调
 * @param panel 页面
 * @returns void
 */
function panelOnLeave(panel) {
    try {
        // 关闭页面时 只有正在进行评估的任务，才调用后端停止评估命令
        if (panel.stored.evaluationStatus === 'evaluating') {
            // 如果离开评估页，需要中止评估
            panel.stored.conn && panel.stored.conn.exeCMDRemote(global_constants_1.HALT_EXECS);
        }
        // 如果离开部署页，需要清理临时工作目录
        panel.stored.deploy && panel.stored.deploy.afterDeploy();
    }
    catch (e) {
        // 失败也没事
    }
}
exports.panelOnLeave = panelOnLeave;
//# sourceMappingURL=util.js.map