"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.WebviewProvider = void 0;
const vscode = require("vscode");
const fs = require("fs");
const path = require("path");
const vscode_api_1 = require("../vscode_api");
const vscode_api_2 = require("../vscode_api/vscode_api");
const global_constants_1 = require("../consts/global_constants");
const ssh2tools_1 = require("../ssh2tool/ssh2tools");
const msg_handler_1 = require("../msg_manager/msg_handler");
const base_util_1 = require("../utils/base_util");
const util_1 = require("./util");
const HandleWebviewEvents_1 = require("../msg_manager/HandleWebviewEvents");
const server_manage_1 = require("../server_manage");
// 单例
let instance;
let conn;
let htmlString;
const whiteList = [/deploy\/server/, /deploy\/easy/, /deploy\/tool/, /report-list/];
/**
 * Webview类
 * 单例
 * 包含对webview增删改查的操作
 * 可以进行链式调用
 * @param $context: vscode扩展上下文
 * @param $panels: 包含已创建的webviewPanels的数组
 */
class WebviewProvider {
    constructor() {
        if (instance) {
            return instance;
        }
        else {
            instance = this;
            this.$panels = [];
            this.$id = 1;
        }
    }
    /**
     * 定义页面当前ssh连接信息
     * @param panel 页面
     * @param serverMark 连接的ssh的服务器和端口信息 host:port字符串
     * @returns void
     */
    static defPanelStored(panel, serverMark) {
        panel.stored.serverMark = serverMark;
        if (panel.stored.conn)
            return;
        Object.defineProperty(panel.stored, 'conn', {
            get() {
                return ssh2tools_1.SSH2Client.getConnByMark(this.serverMark);
            }
        });
    }
    // 返回当前id
    getId() {
        return this.$id;
    }
    // 按title找到对应的panel
    getPanelByTitle(title) {
        return this.$panels.find(e => e.view.title === title);
    }
    // 按visible找到对应的panel
    getPanelByVisible() {
        return this.$panels.find(e => e.view.visible);
    }
    // 按id找到对应的panel
    getPanelById(id) {
        return this.$panels.find(e => e.id === id);
    }
    // 判断当前是否有panel
    hasPanel() {
        return this.$panels.length > 0;
    }
    // 把html渲染给对应的panel
    renderWebview(panel) {
        panel.view.webview.html = panel.html;
        return this;
    }
    // 根据title返回所有符合的panel
    getPanelsByTitle(title) {
        let panelArray = [];
        this.$panels.map(item => {
            if (item.view.title === title) {
                panelArray.push(item);
            }
        });
        return panelArray;
    }
    // 更新webview的状态：可包括标题，html内容
    updateWebview(options, panel) {
        if (options.title) {
            panel.view.title = options.title;
        }
        if (options.html) {
            panel.view.webview.html = options.html;
        }
        return this;
    }
    // 新增一个panel
    appendWebview(settings) {
        const viewType = settings.viewType || settings.title;
        const title = settings.title || global_constants_1.PANEL_TITLE['deploy/server'];
        const showOptions = settings.showOptions || vscode.ViewColumn.One;
        const options = settings.options || {
            // webview 面板的内容配置
            enableScripts: true,
            retainContextWhenHidden: true,
        };
        const onDestroy = settings.onDestroy || (() => { });
        const onChange = settings.onChange || (() => { });
        const id = this.$id++;
        const panelOptions = {
            viewType,
            title,
            showOptions,
            options,
            onDestroy,
            onChange
        };
        const panel = vscode_api_1.default.createPanel(panelOptions.viewType, panelOptions.title, panelOptions.showOptions, panelOptions.options);
        panel.onDidDispose((e) => {
            panelOptions.onDestroy(e);
            this.removeWebview(id);
        });
        panel.onDidChangeViewState((e) => {
            panelOptions.onChange(e);
        });
        const vpanel = util_1.createVpanel(id, panel);
        // 将panel放到一个有序表中进行维护
        this.$panels.push(vpanel);
        return vpanel;
    }
    // 移除对应id的panel
    removeWebview(id) {
        const index = this.$panels.findIndex(e => e.id === id);
        if (index !== -1) {
            this.$panels.splice(index, 1);
        }
        return this;
    }
    // 为panel设置html
    setCurrenHTML(html, panel, reg = false) {
        if (!reg) {
            html = html.replace(/(src|href)="(.*?)"/g, (m, $1, $2) => {
                const p = path.join(vscode_api_1.default.extensionPath, 'out', 'webview', $2);
                if (/\.js$/.test($2)) {
                    const jsFile = fs.readFileSync(p, 'utf-8');
                    fs.writeFileSync(p, jsFile.replace(/\w\.\w\+"(.*?)"/g, (m, $1) => {
                        const absPath = path.join(vscode_api_1.default.extensionPath, 'out', 'webview', $1);
                        return `"${util_1.asWebviewUri(panel, util_1.vscodeUri(absPath))}"`;
                    }));
                }
                return `${$1}="${util_1.asWebviewUri(panel, util_1.vscodeUri(p))}"`;
            });
        }
        panel.html = html;
        return html;
    }
    // 查看是否有重复id的panel
    idDuplicate(id) {
        return this.$panels.some(panel => panel.id === id);
    }
    // 关闭满足条件的页面
    closeViewByID(filter) {
        const trash = [];
        const updateArr = [];
        this.$panels.forEach(panel => {
            filter(panel)
                ? trash.push(panel)
                : updateArr.push(panel);
        });
        this.$panels = updateArr;
        trash.forEach(panel => panel.view.dispose());
    }
    newTab(page = 'deploy/server', isNew = false, data) {
        return __awaiter(this, void 0, void 0, function* () {
            let panel = this.getPanelByTitle(global_constants_1.PANEL_TITLE[page]);
            conn = ssh2tools_1.SSH2Client.getConnByMark(server_manage_1.serverManage.active.serverMark);
            const server = conn && conn.getServerInfo();
            const serverAccessable = conn && conn.hasConnected2Server();
            let flag = true;
            if (panel && !isNew) {
                panel.view.reveal();
            }
            else {
                if (!htmlString) {
                    flag = false;
                    htmlString = yield util_1.getHTML();
                }
                panel = this
                    .appendWebview({
                    viewType: page,
                    title: global_constants_1.PANEL_TITLE[page],
                    onChange: (e) => __awaiter(this, void 0, void 0, function* () {
                        util_1.panelOnShow(panel, page);
                    }),
                    onDestroy: () => {
                        util_1.panelOnLeave(panel);
                    }
                });
                msg_handler_1.registerEvent(panel, {
                    pagePrepared: () => {
                        this.gotoPage(serverAccessable, page, panel, server, data);
                    }
                });
                htmlString = this.setCurrenHTML(htmlString, panel, flag);
                this.renderWebview(panel);
                util_1.panelOnShow(panel, page);
            }
            // 如果创建新页面时已有连接，则默认将该连接信息储存在新页面中
            if (conn) {
                const { host, port } = conn.getServerInfo();
                const serverMark = `${host}:${port}`;
                WebviewProvider.defPanelStored(panel, serverMark);
            }
            return panel;
        });
    }
    gotoPage(serverAccessable, page, panel, server, data) {
        // 如果页面不是报告页或者登录成功了或者页面在白名单中，则跳转页面，否则提示未登录
        if (!page.endsWith('result') && (serverAccessable || base_util_1.inWhiteList(page, whiteList))) {
            if (server && !server.username)
                server.username = 'x2openEuler';
            panel.postMsg({
                command: 'change-page',
                data: Object.assign({ path: page, params: {
                        form: server
                    } }, data)
            });
        }
        else if (page.endsWith('result')) {
            this.updateWebview({
                html: data.data.msg,
                title: data.data.download.name
            }, panel);
            let { name, path } = data.data.download;
            HandleWebviewEvents_1.handleReportDownload(panel, { name, path });
        }
        else {
            this.updateWebview({
                title: vscode_api_2.I18n.common_pannel_title.configure_server
            }, panel);
            vscode_api_1.default.errMsg(vscode_api_2.I18n.common_server_error.no_server);
        }
    }
    /**
     * 跳转到获取报告中的页面
     * @param wp webview管理者
     * @param options 包含评估类型信息的对象
     * @param page 要跳转到的页面
     * @returns
     */
    turnToProgress(options, page = 'report/progress') {
        return __awaiter(this, void 0, void 0, function* () {
            const data = {};
            data.estimateType = options.type;
            options.taskType && (data.taskType = options.taskType);
            return this.newTab(page, true, data);
        });
    }
}
exports.WebviewProvider = WebviewProvider;
//# sourceMappingURL=webview_provider.js.map